/*
 * Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Authors: Radek Novacek <rnovacek@redhat.com>
 */

#include <konkret/konkret.h>
#include "LMI_IPNetworkConnection.h"
#include "network.h"
#include "port.h"

static const CMPIBroker* _cb = NULL;

static void LMI_IPNetworkConnectionInitialize(CMPIInstanceMI *mi)
{
    mi->hdl = network_ref(_cb);
}

static CMPIStatus LMI_IPNetworkConnectionCleanup(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    network_unref(mi->hdl);
    mi->hdl = NULL;
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus LMI_IPNetworkConnectionEnumInstanceNames(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    return KDefaultEnumerateInstanceNames(
        _cb, mi, cc, cr, cop);
}

static CMPIStatus LMI_IPNetworkConnectionEnumInstances(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    CMPIStatus res = { CMPI_RC_OK, NULL };
    Network *network = mi->hdl;
    const char *ns = KNameSpace(cop);

    Port *port;
    network_lock(network);
    const Ports *ports = network_get_ports(network);
    for (size_t i = 0; i < ports_length(ports); ++i) {
        port = ports_index(ports, i);

        LMI_IPNetworkConnection w;
        LMI_IPNetworkConnection_Init(&w, _cb, ns);

        LMI_IPNetworkConnection_Set_CreationClassName(&w, LMI_IPNetworkConnection_ClassName);
        LMI_IPNetworkConnection_Set_Name(&w, port_get_id(port));
        LMI_IPNetworkConnection_Set_SystemCreationClassName(&w, get_system_creationg_class_name());
        LMI_IPNetworkConnection_Set_SystemName(&w, get_system_name());

        switch (port_get_operating_status(port)) {
            case STATUS_NA:
                LMI_IPNetworkConnection_Set_OperatingStatus(&w, LMI_IPNetworkConnection_OperatingStatus_Not_Available);
                break;
            case STATUS_IN_SERVICE:
                LMI_IPNetworkConnection_Set_OperatingStatus(&w, LMI_IPNetworkConnection_OperatingStatus_In_Service);
                break;
            case STATUS_STARTING:
                LMI_IPNetworkConnection_Set_OperatingStatus(&w, LMI_IPNetworkConnection_OperatingStatus_Starting);
                break;
            case STATUS_STOPPING:
                LMI_IPNetworkConnection_Set_OperatingStatus(&w, LMI_IPNetworkConnection_OperatingStatus_Stopping);
                break;
            case STATUS_STOPPED:
                LMI_IPNetworkConnection_Set_OperatingStatus(&w, LMI_IPNetworkConnection_OperatingStatus_Stopped);
                break;
            case STATUS_OFFLINE:
                LMI_IPNetworkConnection_Set_OperatingStatus(&w, LMI_IPNetworkConnection_OperatingStatus_Dormant);
                break;
            case STATUS_ABORTED:
                LMI_IPNetworkConnection_Set_OperatingStatus(&w, LMI_IPNetworkConnection_OperatingStatus_Aborted);
                break;
            default:
                LMI_IPNetworkConnection_Set_OperatingStatus(&w, LMI_IPNetworkConnection_OperatingStatus_Unknown);
        }

        LMI_IPNetworkConnection_Set_ID(&w, port_get_id(port));

        if (!ReturnInstance(cr, w)) {
            error("Unable to return instance of class " LMI_IPNetworkConnection_ClassName);
            CMSetStatus(&res, CMPI_RC_ERR_FAILED);
            break;
        }
    }
    network_unlock(network);
    return res;
}

static CMPIStatus LMI_IPNetworkConnectionGetInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    return KDefaultGetInstance(
        _cb, mi, cc, cr, cop, properties);
}

static CMPIStatus LMI_IPNetworkConnectionCreateInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus LMI_IPNetworkConnectionModifyInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci,
    const char** properties)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus LMI_IPNetworkConnectionDeleteInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus LMI_IPNetworkConnectionExecQuery(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* lang,
    const char* query)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

CMInstanceMIStub(
    LMI_IPNetworkConnection,
    LMI_IPNetworkConnection,
    _cb,
    LMI_IPNetworkConnectionInitialize(&mi))

static void LMI_IPNetworkConnectionMethodInitialize(CMPIMethodMI *mi)
{
    mi->hdl = network_ref(_cb);
}

static CMPIStatus LMI_IPNetworkConnectionMethodCleanup(
    CMPIMethodMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    network_unref(mi->hdl);
    mi->hdl = NULL;
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus LMI_IPNetworkConnectionInvokeMethod(
    CMPIMethodMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* meth,
    const CMPIArgs* in,
    CMPIArgs* out)
{
    return LMI_IPNetworkConnection_DispatchMethod(
        _cb, mi, cc, cr, cop, meth, in, out);
}

CMMethodMIStub(
    LMI_IPNetworkConnection,
    LMI_IPNetworkConnection,
    _cb,
    LMI_IPNetworkConnectionMethodInitialize(&mi))

KUint32 LMI_IPNetworkConnection_RequestStateChange(
    const CMPIBroker* cb,
    CMPIMethodMI* mi,
    const CMPIContext* context,
    const LMI_IPNetworkConnectionRef* self,
    const KUint16* RequestedState,
    KRef* Job,
    const KDateTime* TimeoutPeriod,
    CMPIStatus* status)
{
    KUint32 result = KUINT32_INIT;

    KSetStatus(status, ERR_NOT_SUPPORTED);
    return result;
}

KONKRET_REGISTRATION(
    "root/cimv2",
    "LMI_IPNetworkConnection",
    "LMI_IPNetworkConnection",
    "instance method")
