/*
 * Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Authors: Radek Novacek <rnovacek@redhat.com>
 */

#include "globals.h"

#include <konkret/konkret.h>
#include "LMI_LANEndpoint.h"

#include "network.h"
#include "port.h"
#include "errors.h"

#include <string.h>

static const CMPIBroker* _cb = NULL;

static void LMI_LANEndpointInitialize(CMPIInstanceMI *mi)
{
    mi->hdl = network_ref(_cb);
}

static CMPIStatus LMI_LANEndpointCleanup(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    network_unref(mi->hdl);
    mi->hdl = NULL;
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus LMI_LANEndpointEnumInstanceNames(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    return KDefaultEnumerateInstanceNames(
        _cb, mi, cc, cr, cop);
}

static CMPIStatus LMI_LANEndpointEnumInstances(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    CMPIStatus res = { CMPI_RC_OK, NULL };
    Network *network = mi->hdl;
    const char *ns = KNameSpace(cop);

    Port *port;
    network_lock(network);
    const Ports *ports = network_get_ports(network);
    for (size_t i = 0; i < ports_length(ports); ++i) {
        port = ports_index(ports, i);

        LMI_LANEndpoint w;
        LMI_LANEndpoint_Init(&w, _cb, ns);

        LMI_LANEndpoint_Set_CreationClassName(&w, LMI_LANEndpoint_ClassName);
        LMI_LANEndpoint_Set_Name(&w, port_get_id(port));
        LMI_LANEndpoint_Set_SystemCreationClassName(&w, get_system_creationg_class_name());
        LMI_LANEndpoint_Set_SystemName(&w, get_system_name());

        LMI_LANEndpoint_Set_MACAddress(&w, port_get_mac(port));
        LMI_LANEndpoint_Null_NameFormat(&w);
        LMI_LANEndpoint_Set_ElementName(&w, port_get_id(port));
        LMI_LANEndpoint_Set_ProtocolIFType(&w, LMI_LANEndpoint_ProtocolIFType_Ethernet_CSMA_CD);

        // OperatingStatus provides a current status value for the operational
        // condition of the element and can be used for providing more detail
        // with respect to the value of EnabledState
        switch (port_get_operating_status(port)) {
            case STATUS_NA:
                LMI_LANEndpoint_Set_OperatingStatus(&w, LMI_LANEndpoint_OperatingStatus_Not_Available);
                break;
            case STATUS_OFFLINE:
                LMI_LANEndpoint_Set_OperatingStatus(&w, LMI_LANEndpoint_OperatingStatus_Dormant);
                break;
            case STATUS_IN_SERVICE:
                LMI_LANEndpoint_Set_OperatingStatus(&w, LMI_LANEndpoint_OperatingStatus_In_Service);
                break;
            case STATUS_STARTING:
                LMI_LANEndpoint_Set_OperatingStatus(&w, LMI_LANEndpoint_OperatingStatus_Starting);
                break;
            case STATUS_STOPPING:
                LMI_LANEndpoint_Set_OperatingStatus(&w, LMI_LANEndpoint_OperatingStatus_Stopping);
                break;
            case STATUS_STOPPED:
                LMI_LANEndpoint_Set_OperatingStatus(&w, LMI_LANEndpoint_OperatingStatus_Stopped);
                break;
            case STATUS_ABORTED:
                LMI_LANEndpoint_Set_OperatingStatus(&w, LMI_LANEndpoint_OperatingStatus_Aborted);
                break;
            default:
                LMI_LANEndpoint_Set_OperatingStatus(&w, LMI_LANEndpoint_OperatingStatus_Unknown);
        }

        // AvailableRequestedStates indicates the possible values for the
        // RequestedState parameter of the method RequestStateChange, used to
        // initiate a state change.
        LMI_LANEndpoint_Init_AvailableRequestedStates(&w, 2);
        LMI_LANEndpoint_Set_AvailableRequestedStates(&w, 0, LMI_LANEndpoint_AvailableRequestedStates_Enabled);
        LMI_LANEndpoint_Set_AvailableRequestedStates(&w, 1, LMI_LANEndpoint_AvailableRequestedStates_Disabled);

        // RequestedState is an integer enumeration that indicates the last
        // requested or desired state for the element, irrespective of
        // the mechanism through which it was requested.
        switch (port_get_requested_state(port)) {
            case STATE_DISABLED:
                LMI_LANEndpoint_Set_RequestedState(&w, LMI_LANEndpoint_RequestedState_Disabled);
                break;
            case STATE_ENABLED:
                LMI_LANEndpoint_Set_RequestedState(&w, LMI_LANEndpoint_RequestedState_Enabled);
                break;
            case STATE_NO_CHANGE:
                LMI_LANEndpoint_Set_RequestedState(&w, LMI_LANEndpoint_RequestedState_No_Change);
                break;
            default:
                LMI_LANEndpoint_Set_RequestedState(&w, LMI_LANEndpoint_RequestedState_Unknown);
                break;
        }

        switch (port_get_enabled_state(port)) {
            case STATE_ENABLED:
                LMI_LANEndpoint_Set_EnabledState(&w, LMI_LANEndpoint_EnabledState_Enabled);
                break;
            case STATE_DISABLED:
                LMI_LANEndpoint_Set_EnabledState(&w, LMI_LANEndpoint_EnabledState_Disabled);
                break;
            case STATE_ENABLED_BUT_OFFLINE:
                LMI_LANEndpoint_Set_EnabledState(&w, LMI_LANEndpoint_EnabledState_Enabled_but_Offline);
                break;
            default:
                LMI_LANEndpoint_Set_EnabledState(&w, LMI_LANEndpoint_EnabledState_Unknown);
        }

        if (!ReturnInstance(cr, w)) {
            error("Unable to return instance of class " LMI_LANEndpoint_ClassName);
            CMSetStatus(&res, CMPI_RC_ERR_FAILED);
            break;
        }
    }
    network_unlock(network);
    return res;
}

static CMPIStatus LMI_LANEndpointGetInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    return KDefaultGetInstance(
        _cb, mi, cc, cr, cop, properties);
}

static CMPIStatus LMI_LANEndpointCreateInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus LMI_LANEndpointModifyInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci,
    const char** properties)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus LMI_LANEndpointDeleteInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus LMI_LANEndpointExecQuery(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* lang,
    const char* query)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

CMInstanceMIStub(
    LMI_LANEndpoint,
    LMI_LANEndpoint,
    _cb,
    LMI_LANEndpointInitialize(&mi))

static void LMI_LANEndpointMethodInitialize(CMPIMethodMI *mi)
{
    mi->hdl = network_ref(_cb);
}

static CMPIStatus LMI_LANEndpointMethodCleanup(
    CMPIMethodMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    network_unref(mi->hdl);
    mi->hdl = NULL;
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus LMI_LANEndpointInvokeMethod(
    CMPIMethodMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* meth,
    const CMPIArgs* in,
    CMPIArgs* out)
{
    return LMI_LANEndpoint_DispatchMethod(
        _cb, mi, cc, cr, cop, meth, in, out);
}

CMMethodMIStub(
    LMI_LANEndpoint,
    LMI_LANEndpoint,
    _cb,
    LMI_LANEndpointMethodInitialize(&mi))

enum {
    LMI_LANEndpoint_RequestStateChange_No_Error=0,
    LMI_LANEndpoint_RequestStateChange_Not_Supported=1,
    LMI_LANEndpoint_RequestStateChange_Unknown_Error=2,
    LMI_LANEndpoint_RequestStateChange_Failed=4,
    LMI_LANEndpoint_RequestStateChange_Invalid_Parameter=5,
    LMI_LANEndpoint_RequestStateChange_Timeout_Not_Supported=4098
};

KUint32 LMI_LANEndpoint_RequestStateChange(
    const CMPIBroker* cb,
    CMPIMethodMI* mi,
    const CMPIContext* context,
    const LMI_LANEndpointRef* self,
    const KUint16* RequestedState,
    KRef* Job,
    const KDateTime* TimeoutPeriod,
    CMPIStatus* status)
{
    Network *network = mi->hdl;
    KUint32 result = KUINT32_INIT;

    if (TimeoutPeriod->exists && !TimeoutPeriod->null) {
        KSetStatus2(_cb, status, ERR_NOT_SUPPORTED, "Use of Timeout Parameter Not Supported");
        KUint32_Set(&result, LMI_LANEndpoint_RequestStateChange_Timeout_Not_Supported);
        return result;
    }

    Require(RequestedState, "No state has been requested", result, LMI_LANEndpoint_RequestStateChange_Invalid_Parameter);

    network_lock(network);
    Port *port = NULL;
    const Ports *ports = network_get_ports(network);
    for (size_t i = 0; i < ports_length(ports); ++i) {
        if (strcmp(port_get_id(ports_index(ports, i)), self->Name.chars) == 0) {
            port = ports_index(ports, i);
        }
    }
    if (port == NULL) {
        KSetStatus2(_cb, status, ERR_INVALID_PARAMETER, "No such " LMI_LANEndpoint_ClassName);
        KUint32_Set(&result, LMI_LANEndpoint_RequestStateChange_Invalid_Parameter);
        network_unlock(network);
        return result;
    }

    int res = LMI_SUCCESS;
    switch (RequestedState->value) {
        case LMI_LANEndpoint_RequestedState_Disabled:
            res = port_request_state(port, STATE_DISABLED);
            break;
        case LMI_LANEndpoint_RequestedState_Enabled:
            res = port_request_state(port, STATE_ENABLED);
            break;
        default:
            KSetStatus2(_cb, status, ERR_INVALID_PARAMETER, "Invalid state requested");
            KUint32_Set(&result, LMI_LANEndpoint_RequestStateChange_Invalid_Parameter);
            network_unlock(network);
            return result;
    }
    network_unlock(network);

    if (res != LMI_SUCCESS) {
        KSetStatus2(_cb, status, ERR_FAILED, "Unable to set state");
        KUint32_Set(&result, LMI_LANEndpoint_RequestStateChange_Failed);
        return result;
    }

    KSetStatus(status, OK);
    KUint32_Set(&result, LMI_LANEndpoint_RequestStateChange_No_Error);
    return result;
}

KUint32 LMI_LANEndpoint_BroadcastReset(
    const CMPIBroker* cb,
    CMPIMethodMI* mi,
    const CMPIContext* context,
    const LMI_LANEndpointRef* self,
    CMPIStatus* status)
{
    KUint32 result = KUINT32_INIT;

    KSetStatus(status, ERR_NOT_SUPPORTED);
    return result;
}

KONKRET_REGISTRATION(
    "root/cimv2",
    "LMI_LANEndpoint",
    "LMI_LANEndpoint",
    "instance method")
