/*
 * Copyright (C) 2013 Red Hat, Inc.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Authors: Radek Novacek <rnovacek@redhat.com>
 */

#include "ipassignmentsettingdata.h"
#include "connection.h"
#include "setting.h"
#include <string.h>

#include "LMI_IPAssignmentSettingData.h"
#include "LMI_IPRouteSettingData.h"
#include "LMI_ExtendedStaticIPAssignmentSettingData.h"
#include "LMI_DHCPSettingData.h"

CMPIStatus IPAssignmentSettingDataEnumInstances(
    const char *classname,
    Network *network,
    const CMPIResult* cr,
    const CMPIBroker* cb,
    const char *ns)
{
    CMPIStatus res = { CMPI_RC_OK, NULL };
    size_t j, k, count;
    Connection *connection;
    char *instanceid, *id, *name, *mask;
    Setting *setting;
    Addresses *addresses;
    Address *address;
    Route *route;

    network_lock(network);

    const Connections *connections = network_get_connections(network);
    for (size_t i = 0; i < connections_length(connections); ++i) {
        connection = connections_index(connections, i);

        if (strcmp(classname, LMI_IPAssignmentSettingData_ClassName) == 0) {
            // Cumulative IPAssignmentSettingData
            LMI_IPAssignmentSettingData w;
            LMI_IPAssignmentSettingData_Init(&w, cb, ns);
            LMI_IPAssignmentSettingData_Set_Caption(&w, connection_get_name(connection));
            instanceid = id_to_instanceid(connection_get_id(connection), LMI_IPAssignmentSettingData_ClassName);
            if (instanceid == NULL) {
                error("Memory allocation failed");
                CMSetStatus(&res, CMPI_RC_ERR_FAILED);
                break;
            }
            LMI_IPAssignmentSettingData_Set_InstanceID(&w, instanceid);
            free(instanceid);
            LMI_IPAssignmentSettingData_Set_AddressOrigin(&w,
                    LMI_IPAssignmentSettingData_AddressOrigin_cumulative_configuration);
            LMI_IPAssignmentSettingData_Set_ProtocolIFType(&w,
                    LMI_IPAssignmentSettingData_ProtocolIFType_Both_IPv4_and_IPv6);

            if (!ReturnInstance(cr, w)) {
                error("Unable to return instance of class " LMI_ExtendedStaticIPAssignmentSettingData_ClassName);
                CMSetStatus(&res, CMPI_RC_ERR_FAILED);
                break;
            }
        }

        for (j = 0; j < settings_length(connection_get_settings(connection)); ++j) {
            setting = settings_index(connection_get_settings(connection), j);

            if (setting_get_type(setting) != SETTING_TYPE_IPv4 &&
                setting_get_type(setting) != SETTING_TYPE_IPv6) {

                continue;
            }

            if (setting_get_method(setting) == SETTING_METHOD_STATIC &&
                strcmp(classname, LMI_ExtendedStaticIPAssignmentSettingData_ClassName) == 0) {

                LMI_ExtendedStaticIPAssignmentSettingData w;
                LMI_ExtendedStaticIPAssignmentSettingData_Init(&w, cb, ns);
                LMI_ExtendedStaticIPAssignmentSettingData_Set_Caption(&w, setting_get_caption(setting));
                instanceid = id_to_instanceid(setting_get_id(setting), LMI_ExtendedStaticIPAssignmentSettingData_ClassName);
                if (instanceid == NULL) {
                    error("Memory allocation failed");
                    CMSetStatus(&res, CMPI_RC_ERR_FAILED);
                    break;
                }
                LMI_ExtendedStaticIPAssignmentSettingData_Set_InstanceID(&w, instanceid);
                free(instanceid);
                LMI_ExtendedStaticIPAssignmentSettingData_Set_ElementName(&w, setting_get_caption(setting));

                LMI_ExtendedStaticIPAssignmentSettingData_Set_ProtocolIFType(&w,
                        setting_get_type(setting) == SETTING_TYPE_IPv4 ?
                        LMI_ExtendedStaticIPAssignmentSettingData_ProtocolIFType_IPv4 :
                        LMI_ExtendedStaticIPAssignmentSettingData_ProtocolIFType_IPv6);
                addresses = setting_get_addresses(setting);

                count = addresses_length(addresses);
                LMI_ExtendedStaticIPAssignmentSettingData_Init_IPAddresses(&w, count);
                if (setting_get_type(setting) == SETTING_TYPE_IPv4) {
                    LMI_ExtendedStaticIPAssignmentSettingData_Init_SubnetMasks(&w, count);
                } else {
                    LMI_ExtendedStaticIPAssignmentSettingData_Init_IPv6SubnetPrefixLengths(&w, count);
                }
                LMI_ExtendedStaticIPAssignmentSettingData_Init_GatewayAddresses(&w, count);

                for (k = 0; k < count; ++k) {
                    address = addresses_index(addresses, k);
                    LMI_ExtendedStaticIPAssignmentSettingData_Set_IPAddresses(&w, k, address->addr);
                    if (setting_get_type(setting) == SETTING_TYPE_IPv4) {
                        LMI_ExtendedStaticIPAssignmentSettingData_Set_SubnetMasks(&w, k, prefixToMask4(address->prefix));
                    } else {
                        LMI_ExtendedStaticIPAssignmentSettingData_Set_IPv6SubnetPrefixLengths(&w, k, address->prefix);
                    }
                    if (address->default_gateway != NULL) {
                        LMI_ExtendedStaticIPAssignmentSettingData_Set_GatewayAddresses(&w, k, address->default_gateway);
                    } else {
                        LMI_ExtendedStaticIPAssignmentSettingData_Null_GatewayAddresses(&w, k);
                    }
                }

                if (!ReturnInstance(cr, w)) {
                    error("Unable to return instance of class " LMI_ExtendedStaticIPAssignmentSettingData_ClassName);
                    CMSetStatus(&res, CMPI_RC_ERR_FAILED);
                    break;
                }
            }

            if ((setting_get_method(setting) == SETTING_METHOD_DHCP ||
                 setting_get_method(setting) == SETTING_METHOD_DHCPv6) &&
                strcmp(classname, LMI_DHCPSettingData_ClassName) == 0) {

                LMI_DHCPSettingData w;
                LMI_DHCPSettingData_Init(&w, cb, ns);

                // AddressOrigin has the same number as SettingMethod
                LMI_DHCPSettingData_Set_AddressOrigin(&w, setting_get_method(setting));
                LMI_DHCPSettingData_Set_Caption(&w, setting_get_caption(setting));
                LMI_DHCPSettingData_Set_ProtocolIFType(&w,
                        setting_get_method(setting) == SETTING_METHOD_DHCP ?
                        LMI_DHCPSettingData_ProtocolIFType_IPv4 :
                        LMI_DHCPSettingData_ProtocolIFType_IPv6);

                instanceid = id_to_instanceid(setting_get_id(setting), LMI_DHCPSettingData_ClassName);
                if (instanceid == NULL) {
                    error("Memory allocation failed");
                    CMSetStatus(&res, CMPI_RC_ERR_FAILED);
                    break;
                }
                LMI_DHCPSettingData_Set_InstanceID(&w, instanceid);
                free(instanceid);

                if (!ReturnInstance(cr, w)) {
                    error("Unable to return instance of class " LMI_DHCPSettingData_ClassName);
                    CMSetStatus(&res, CMPI_RC_ERR_FAILED);
                    break;
                }
            }

            if (setting_get_method(setting) == SETTING_METHOD_STATELESS &&
                strcmp(classname, LMI_IPAssignmentSettingData_ClassName) == 0) {

                LMI_IPAssignmentSettingData w;
                LMI_IPAssignmentSettingData_Init(&w, cb, ns);

                // AddressOrigin has the same number as SettingMethod
                LMI_IPAssignmentSettingData_Set_AddressOrigin(&w, setting_get_method(setting));
                LMI_IPAssignmentSettingData_Set_Caption(&w, setting_get_caption(setting));
                LMI_IPAssignmentSettingData_Set_ProtocolIFType(&w, LMI_IPAssignmentSettingData_ProtocolIFType_IPv6);

                instanceid = id_to_instanceid(setting_get_id(setting), LMI_IPAssignmentSettingData_ClassName);
                if (instanceid == NULL) {
                    error("Memory allocation failed");
                    CMSetStatus(&res, CMPI_RC_ERR_FAILED);
                    break;
                }
                LMI_IPAssignmentSettingData_Set_InstanceID(&w, instanceid);
                free(instanceid);

                if (!ReturnInstance(cr, w)) {
                    error("Unable to return instance of class " LMI_IPAssignmentSettingData_ClassName);
                    CMSetStatus(&res, CMPI_RC_ERR_FAILED);
                    break;
                }
            }

            if (strcmp(classname, LMI_IPRouteSettingData_ClassName) == 0) {
                LMI_IPRouteSettingData w;
                LMI_IPRouteSettingData_Init(&w, cb, ns);

                for (k = 0; k < setting_get_routes_length(setting); ++k) {
                    route = setting_get_route(setting, k);

                    if (asprintf(&name, "%s_%ld", setting_get_id(setting), k) < 0) {
                        error("Memory allocation failed");
                        CMSetStatus(&res, CMPI_RC_ERR_FAILED);
                        break;
                    }
                    id = id_to_instanceid(name, "LMI_IPRouteSettingData");
                    if (id == NULL) {
                        error("Unable to get ID from InstanceID: %s", name);
                        CMSetStatus(&res, CMPI_RC_ERR_FAILED);
                        break;
                    }
                    LMI_IPRouteSettingData_Set_InstanceID(&w, id);
                    free(name);
                    free(id);

                    LMI_IPRouteSettingData_Null_AddressOrigin(&w);

                    LMI_IPRouteSettingData_Set_DestinationAddress(&w, route->route);
                    if (route->type == IPv4) {
                        if ((mask = prefixToMask4(route->prefix)) == NULL) {
                            error("Memory allocation failed");
                            CMSetStatus(&res, CMPI_RC_ERR_FAILED);
                            break;
                        }
                        LMI_IPRouteSettingData_Set_DestinationMask(&w, mask);
                        free(mask);
                    } else {
                        LMI_IPRouteSettingData_Set_PrefixLength(&w, route->prefix);
                    }
                    LMI_IPRouteSettingData_Set_RouteMetric(&w, route->metric);
                    LMI_IPRouteSettingData_Set_AddressType(&w,
                    route->type == IPv4 ?
                            LMI_IPRouteSettingData_AddressType_IPv4 :
                            LMI_IPRouteSettingData_AddressType_IPv6);
                    LMI_IPRouteSettingData_Set_NextHop(&w, route->next_hop);

                    if (!ReturnInstance(cr, w)) {
                        error("Unable to return instance of class " LMI_IPRouteSettingData_ClassName);
                        CMSetStatus(&res, CMPI_RC_ERR_FAILED);
                        break;
                    }
                }
            }
        }
    }

    network_unlock(network);
    return res;
}
